# bloop `v1.3.0`

Bloop is a [build server and CLI tool for the Scala programming language]() developed by the Scala Center.

Bloop `v1.3.0` is the biggest release of this project to this date.

With over 520 commits since the `v1.2.5` release, ~26000 additions and ~9500
deletions, it aggregates together fundamental changes in the compilation
pipeline, novel build server features, user-facing improvements and bug fixes
together.

## Upgrade guide :inbox_tray:

These are **only** the upgrade steps. If you don't have bloop installed, you must read
the [installation instructions][] instead.

If you're on macOS, **upgrade** to the latest version with:

```sh
$ brew upgrade scalacenter/bloop/bloop
$ brew services restart bloop # Note a restart is required in case a server still runs!
```

If you're on Windows using `scoop`, **upgrade** to the latest version with:

```sh
$ scoop upgrade bloop
$ bloop ng-stop
$ bloop server # in a long-running terminal session
```

:new: If you're on Arch Linux, **upgrade** to the latest version with:

```sh
$ rm -rf ~/.bloop
$ yaourt -S bloop
$ systemctl --user start bloop
```

Otherwise, use the generic installation method:

```
$ curl -L https://github.com/scalacenter/bloop/releases/download/v1.3.0/install.py | python
$ bloop ng-stop
$ systemctl --user start bloop # or similar way to run server in a long-running session
```

Read the complete installation instructions in our [Installation page][installation instructions].

## New compilation engine with client guarantees :muscle: :family:

`v1.3.0` features a redesigned compilation engine built to solve tooling
challenges already present in our current developer workflow but brought to
the foreground by the formalized use of a client-server architecture. This
compilation engine sports a more efficient way of doing IO work (with no
compromises in correctness) and provides a new contract to build clients.

Bloop `v1.3.0` and onwards guarantees clients that their actions will have
the smallest usage footprint possible and will not conflict with those of
other concurrent clients being served by the same server in the same build.

For example, if [Metals] is compiling your project via Bloop and you spawn a
bloop CLI command such as `bloop test foo --watch` at the same time, Bloop
guarantees that:

1. The server heavily caches compilations for the same inputs (aka *compile deduplication*)
   > If inputs haven't changed between the requests, only the first client
   > request will trigger a compilation. The compilation of the second client
   > will be deduplicated based on the compilation side effects recorded by the
   >build server, so only one compilation will happen.
2. Different compilation requests in the same build can run concurrently (aka
   *compile isolation*)
   > If inputs have changed between requests, Bloop will compile the changed
   > projects concurrently, avoiding shared state and conflicts with ongoing
   > compilations.
3. The outputs produced by both requests are independent in the file system.
   > The compilation products will be stored in independent target
   > directories only owned by the client. This independence is essential to
   > allow clients to independently run any build action without altering task
   > executions. Think avoiding crashing JVMs because other processes are
   > writing contents to open class file pointers!

This is the first step towards modelling these build actions as pure
functions despite their pervasive side effects. Bloop manages any internal
state as best as possible to provide the best developer experience to end
users. There will be more work in this area in the future!

#### Related pull requests

1. [Isolate and deduplicate compilation and Zipkin tracing support](https://github.com/scalacenter/bloop/pull/832) by [jvican]
1. [Pre-populate client classes dir if empty](https://github.com/scalacenter/bloop/pull/833) by [jvican]
1. [Deduplicate compilation if project inputs change](https://github.com/scalacenter/bloop/pull/837) by [jvican]
1. [Invalidate symbols with forked and local javac](https://github.com/scalacenter/bloop/pull/845) by [jvican]
1. [Don't create and delete empty classes dirs](https://github.com/scalacenter/bloop/pull/856) by [jvican]
1. [Cancel deduplication if ongoing compilation looks dead](https://github.com/scalacenter/bloop/pull/858) by [jvican]
1. [Use generic classes dir for the pipelined traversal](https://github.com/scalacenter/bloop/pull/860) by [jvican]
1. [Log to the user a deduplication is happening](https://github.com/scalacenter/bloop/pull/861) by [jvican]
1. [Revalidate class files compiled by dependent projects](https://github.com/scalacenter/bloop/pull/881) by [jvican]
1. [Fix not found type errors during incremental compilation](https://github.com/scalacenter/bloop/pull/897) by [jvican]
1. [Add CLI lock timeout and be more flexible when locking CLI clients](https://github.com/scalacenter/bloop/pull/898)
1. [Fix remaining issues around handling of compile products](https://github.com/scalacenter/bloop/pull/916) by [jvican]
1. [Catch exceptions when computing the bundle](https://github.com/scalacenter/bloop/pull/917) by [jvican]

## New [Zipkin](http://zipkin.apache.org) tracing support :satellite:

Bloop `v1.3.0` traces any compilation request to Zipkin if a Zipkin server is
running in the background. Here's a screenshots showing collected compilation
traces for the `akka/akka` project.

![A Zipkin instance with build traces from the akka/akka project](/docs/assets/zipkin-akka-traces-1.png)

And another screenshot showing the top part of a detailed compilation trace.

![A Zipkin instance showing a detaild profile of a build trace from the akka/akka project](/docs/assets/zipkin-akka-traces-2.png)

The zipkin integration is explained in the performance guide below.

## In-depth compiler performance guide :rocket:

The [compiler performance
guide](https://scalacenter.github.io/bloop/docs/performance-guide) is a new
addition to our documentation that explains users how to:

1. Get familiar with the performance characteristics of project builds
1. Compare the performance of Bloop with that of other tools compiling Scala code
1. Modify a complicated build to optimize build times

The guide is rich in details and written to provide pointers to those users
wanting to speed up their workflow with Bloop and optimize their current
build times further.

It is open for any community addition, so if you'd like to add something,
help us improve it!

## Faster build pipelining :horse_racing:

Build pipelining is a compilation performance technique to start the
compilation of modules in a build before their dependencies have finished
compilation.

Build pipelining was merged back in December, 2018 and made available in
bloop `v1.2.1`, making bloop the first Scala developer tool to implement this
technique and support it under an experimental flag.

Bloop `v1.3.0` changes its internal implementation to make it faster. We're
very excited about what this entails for any Scala build in the wild. The
bloop project will publish benchmarks and a concise guide in a follow-up
post, keep tuned! :smile:


## Use Ammonite by default in `bloop console` :video_game:

Contributed by @[torkelrogstad].

Ammonite is a modernized Scala REPL with syntax highlighting, multi-line
editing, the ability to load maven artifacts directly in the REPL, and many
other quality-of-life improvements missing in the default Scala REPL.

Bloop `v1.3.0` defaults on Ammonite every time you run `bloop console` on a
project. Unlike running `amm` in your terminal, the Ammonite integration will
automatically start an Ammonite REPL using the classpath of a project in your
build. That allows you to use all the code defined in that classpath and
experiment with its behavior.

Interested in the implementation details? Have a look at
[Add support for Ammonite REPL by default](https://github.com/scalacenter/bloop/pull/886).

## Better Metals and BSP integration :metal: :100:

BSP support in bloop `v1.3.0` is vastly superior to that in `v1.2.5`. The new
release implements a few non-implemented endpoints, fixes several
non-reported issues in diagnostics and improves connection support and
cancellation capabilities.

The Metals integration is superior to that in `v1.2.5` thanks to this
non-exhaustive list of PRs:

1. [Guarantee cancellation upon sudden BSP client dropouts](https://github.com/scalacenter/bloop/pull/852) by [jvican]
1. [Upgrade to BSP 2.0.0-M4](https://github.com/scalacenter/bloop/pull/902) by [tgodzik]
1. [Implement BSP `buildTarget/scalaMainClasses`](https://github.com/scalacenter/bloop/pull/869) by [marek1840]
1. [Implement BSP `buildTarget/scalaTestClasses`](https://github.com/scalacenter/bloop/pull/899) by [marek1840]
1. [Send out server BSP responses/notifications in order](https://github.com/scalacenter/bloop/pull/854) by [jvican]
1. A few unlisted PRs that casually fix issues in BSP semantics


## Add support for Scala 2.13 :arrow_forward:

Bloop `v1.3.0` finally [adds support for Scala
2.13.x](https://github.com/scalacenter/bloop/pull/907).


## Add support for JDK 9 and 11 :fast_forward:

Contributed by @[er1c], bloop `v1.3.0` supports JDK 9 and JDK 11. If you run
bloop with these Java runtimes, it is recommended to enable the full parallel
GC with `-XX:+UseParallelGC` as compilation is faster.

For this reason, Java 8 still remains the recommended runtime for the time
being and our installers might install it by default. In the future, we might
consider upgrading to a newer Java runtime.

1. [Compile on Java9+](https://github.com/scalacenter/bloop/pull/889) by [er1c]
1. [Add Java9+ reverse-compatible changes](https://github.com/scalacenter/bloop/pull/870) by [er1c]

## New Arch Linux installer :surfer:

Contributed by [TheElectronWill].

Bloop has a new AUR package for Arch Linux that is updated by our release
infrastructure. You can now install bloop in Arch Linux by running `pacaur -S
bloop` or `yaourt -S bloop`.

## Improved file watcher responsiveness :watch: :fire:

bloop `v1.3.0` has better and snappier file watcher infrastructure, with
[protection against overflowing
events](https://github.com/scalacenter/bloop/pull/863) and [several fixes to
bugs reported after `v1.2.5`](https://github.com/scalacenter/bloop/pull/891).
One of these bugs is a recurrent problems in macOS machines where Bloop
server's JVM would crash due to uncleaned state in the Carbon-based file
watcher.

Support for file watching in `run` commands has not yet been implemented. If
you want to help, drop by [bloop's Gitter channel][gitter] and we will give
you pointers to help implement it.

## Fix use of test arguments in `bloop test` :chart_with_upwards_trend:

Running `bloop test` in your project or passing specific test arguments to
Bloop's test runner could [produce issues such as this one](https://github.com/scalacenter/bloop/issues/906):

```
[E] Unexpected error during remote test execution: 'Socket closed'.
[E] Internal error when running tests: sbt.ForkMain$Run$RunAborted: java.net.SocketException: Broken pipe (Write failed)
```

[This pull request](https://github.com/scalacenter/bloop/pull/843) makes sure
that we handling missing test frameworks better and that the above issues
processing test arguments are gone.

## Improvements in build tools support (`bloopInstall`) :hammer_and_wrench:

### Improvements in `sbt`

#### Automatic support for integration tests in sbt builds

`sbt-bloop` v1.3.0 will [automatically export integration test
configurations](https://github.com/scalacenter/bloop/pull/873) to independent
build targets. If you have modified your sbt build to support integration tests
in your build, you can safely remove that logic.

#### Support sbt source dependencies by default

Contributed by [olafurpg]. Support for sbt project references was before
hidden behind an specific sbt boolean setting. [Now it's enabled by
default](https://github.com/scalacenter/bloop/pull/816).

#### Other

1. [Avoid duplication of projects in `dependencies` field](https://github.com/scalacenter/bloop/pull/883) by [tkroman]
1. [Fix small bug when exporting scalac and javac options in sbt](https://github.com/scalacenter/bloop/pull/876) by [jvican]

### Improvements in `Gradle`

1. [Include Gradle's transitive dependencies in classpath](https://github.com/scalacenter/bloop/pull/818) by [Arthurm1]
1. [Ignore annotation processor in Gradle](https://github.com/scalacenter/bloop/pull/878) by [tgodzik]
1. [Don't add JAR dependencies of Gradle projects in the classpath](https://github.com/scalacenter/bloop/pull/875) by [tgodzik]
1. [Export source dependencies from Gradle](https://github.com/scalacenter/bloop/pull/849) by [tgodzik]

Thanks to these improvements, Metals will be able to support "Import Build"
in Gradle projects in its next release.

### Improvements in `Mill`

1. [Point Mill docs to merged bloop integration upstream](https://github.com/scalacenter/bloop/pull/872) by [Baccata]
1. [Fix broken link in Mill documentation](https://github.com/scalacenter/bloop/pull/828) by [perryao]
1. [Update Mill version](https://github.com/scalacenter/bloop/pull/847) by [robby-phd]

Thanks to these improvements, Metals will be able to support "Import Build"
in mill builds in its next release.

### Improvements in `Maven`

1. [Avoid NPE when collecting resources in Maven's Bloop plugin](https://github.com/scalacenter/bloop/pull/839) by [artyomboyko]
1. [Add source dependencies to Maven plugin](https://github.com/scalacenter/bloop/pull/888) by [tgodzik]
1. [Fix warning about Maven being run in interactive mode](https://github.com/scalacenter/bloop/pull/925) by [tgodzik]
1. [Fix Maven plugin in Windows by adding version of maven-invoker](https://github.com/scalacenter/bloop/pull/900) by [tgodzik]

Thanks to these improvements, Metals will be able to support "Import Build"
in Maven projects in its next release.

## Miscellaneous

Here's a list of key miscellaneous changes in this release.

1. [Synchronize CLI clients to the same build](https://github.com/scalacenter/bloop/pull/864) by [jvican]
1. [Don't recompile project if server dies](https://github.com/scalacenter/bloop/pull/838) by [jvican]
1. [Add `-Dbloop.computation.cores` and env var to configure thread pool](https://github.com/scalacenter/bloop/pull/826) by [jvican]
1. [Display Java version and Java home in `bloop about`](https://github.com/scalacenter/bloop/pull/908) by [cquiroz]
1. [Add failing test case where source files are pulled in multiple times](https://github.com/scalacenter/bloop/pull/919) by [jackkoenig]
1. [Don't print warnings when autocompleting](https://github.com/scalacenter/bloop/pull/910) by [CucumisSativus]
1. [Add buildpress, a tool to automatically export from any build tool to bloop](https://github.com/scalacenter/bloop/pull/879) by [jvican]
1. [Add support for standalone repo path in CLI args](https://github.com/scalacenter/bloop/pull/901) by [tkroman]
1. [Add contributing guide to the website](https://github.com/scalacenter/bloop/pull/884) by [jvican]
1. [Improve bloop launcher's test suite](https://github.com/scalacenter/bloop/pull/836) by [jvican]
1. [Change SCOOP env variable to avoid conflict](https://github.com/scalacenter/bloop/pull/857) by [Arthurm1]
1. [Show correct default for install.py '--dest' option](https://github.com/scalacenter/bloop/pull/815) by [BardurArantsson]
1. [Use a guaranteed unique name for temporary directories](https://github.com/scalacenter/bloop/pull/813) by [propensive]
1. [Update website, enable versioning and assort fixes](https://github.com/scalacenter/bloop/pull/871) by [jvican]
1. [Update contributing guide with expected error](https://github.com/scalacenter/bloop/pull/911) by [CucumisSativus]
1. [Fix installation link in README](https://github.com/scalacenter/bloop/pull/915) by [jrudolph]
1. [Encode UUIDs with Base 64](https://github.com/scalacenter/bloop/pull/851) by [odisseus]
1. [Fix off-one error in fish autocompletions](https://github.com/scalacenter/bloop/pull/874) by [2m]
1. [Use two cores for Bloop's computation pool in Travis](https://github.com/scalacenter/bloop/pull/826) by [jvican]
1. [Update docs URL in Bloop error messages](https://github.com/scalacenter/bloop/pull/841) by [AckoLF]
1. [Update Homebrew server log path](https://github.com/scalacenter/bloop/pull/850) by [regadas]

## Next steps :walking:

The recent work on supporting concurrent build clients, giving strong
building guarantees to clients and the `bloop launcher` released in `v1.2.5`
has crystallized in a robust tooling framework to take our build tool
integrations to the next level.

Bloop `v1.4.0` will enable sbt to offload compilation of projects
automatically to Bloop, speeding up sbt project load and making the
cutting-edge performance improvements in Bloop available to sbt.
The goal is that users will be able to use bloop without changing their
current development workflow.

Such integrations for other build tools will also be possible. The bloop team
volunteers to mentor people interested in contributing such integrations for
popular build tools such as mill or Gradle. If you're interested, please [get
in contact with us][gitter].

## Special acknowledgements :pray:

bloop `v1.3.0` is an exciting release because of the vast amount of external
contributors that we have had. Since `v1.2.5`, bloop has received
contributions by more than 27 OSS contributors.

Additionally, the bloop team has welcomed three new more members: [Tomasz
Godzik][tgodzik] (@tgodzik) and [Marek Żarnowski][marek1840] (@marek1840)
from **VirtusLab** which have made this release a reality and [Roman
Tkalenko][tkroman] (@tkroman) which has recently started contributing in
making Bloop better for CI workflows.

I am thrilled to see the bloop community grow and be working with such
skilled people in improving the tooling in the Scala community.

According to `git shortlog -sn --no-merges v1.2.5..v1.3.0`, 27 people
contributed to this release: Guillaume R, Roman Tkalenko, Jorge,
Eric Peters, Michał Gutowski, Marek Żarnowski, Carlos Quiroz, Artyom Boyko,
Arthur McGibbon, Tomasz Godzik, Torkel Rogstad, Myroslav Golub, AckoLF,
Ólafur Páll Geirsson, ArthurM, Bardur Arantsson, David Gregory, Enver
Osmanov, Filipe Regadas, Johannes Rudolph, Jon Pretty, Martynas Mickevičius,
Mike, Olivier Mélois, Jack Koenig and Robby.

[jvican]: https://github.com/jvican
[Arthurm1]: https://github.com/Arthurm1
[olafurpg]: https://github.com/olafurpg
[propensive]: https://github.com/propensive
[BardurArantsson]: https://github.com/BardurArantsson
[perryao]: https://github.com/perryao
[artyomboyko]: https://github.com/artyomboyko
[AckoLF]: https://github.com/AckoLF
[robby-phd]: https://github.com/robby-phd
[tgodzik]: https://github.com/tgodzik
[regadas]: https://github.com/regadas
[odisseus]: https://github.com/odisseus
[Baccata]: https://github.com/Baccata
[2m]: https://github.com/2m
[torkelrogstad]: https://github.com/torkelrogstad
[cquiroz]: https://github.com/cquiroz
[CucumisSativus]: https://github.com/CucumisSativus
[jrudolph]: https://github.com/jrudolph
[jackkoenig]: https://github.com/jackkoenig
[er1c]: https://github.com/er1c
[marek1840]: https://github.com/marek1840
[tkroman]: https://github.com/tkroman
[TheElectronWill]: https://github.com/TheElectronWill
[gitter]: https://gitter.im/scalacenter/bloop
[installation instructions]: https://scalacenter.github.io/bloop/setup
[Metals]: https://github.com/scalameta/Metals